@@ -2,7 +2,7 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 0, 3)
+__version__ = (0, 0, 5)
 __all__ = [
     "SupportsGeturl", "url_origin", "complete_url", "cookies_str_to_dict", "headers_str_to_dict", 
     "encode_multipart_data", "encode_multipart_data_async", 
@@ -11,12 +11,12 @@ __all__ = [
 from itertools import chain
 from collections.abc import AsyncIterable, AsyncIterator, ItemsView, Iterable, Iterator, Mapping
 from re import compile as re_compile, Pattern
-from typing import Any, Final, Protocol, TypeVar
+from typing import runtime_checkable, Any, Final, Protocol, TypeVar
 from urllib.parse import quote, urlsplit, urlunsplit
 from uuid import uuid4
 
 from asynctools import ensure_aiter, async_chain
-from filewrap import bio_chunk_iter, bio_chunk_async_iter, SupportsRead
+from filewrap import bio_chunk_iter, bio_chunk_async_iter, Buffer, SupportsRead
 from integer_tool import int_to_bytes
 from texttools import text_to_dict
 
@@ -26,6 +26,7 @@ AnyStr = TypeVar("AnyStr", bytes, str, covariant=True)
 CRE_URL_SCHEME: Final = re_compile(r"^(?i:[a-z][a-z0-9.+-]*)://")
 
 
+@runtime_checkable
 class SupportsGeturl(Protocol[AnyStr]):
     def geturl(self) -> AnyStr: ...
 
@@ -72,8 +73,8 @@ def headers_str_to_dict(
     return text_to_dict(headers.strip(), kv_sep, entry_sep)
 
 
-def ensure_bytes(s, /) -> bytes | bytearray | memoryview:
-    if isinstance(s, (bytes, bytearray, memoryview)):
+def ensure_bytes(s, /) -> Buffer:
+    if isinstance(s, Buffer):
         return s
     if isinstance(s, int):
         return int_to_bytes(s)
@@ -86,17 +87,17 @@ def ensure_bytes(s, /) -> bytes | bytearray | memoryview:
 
 
 def encode_multipart_data(
-    data: Mapping[str, Any], 
-    files: Mapping[str, bytes | bytearray | memoryview | 
-                        SupportsRead[bytes] | SupportsRead[bytearray] | SupportsRead[memoryview] | 
-                        Iterable[bytes] | Iterable[bytearray] | Iterable[memoryview]], 
+    data: None | Mapping[str, Any] = None, 
+    files: None | Mapping[str, Buffer | SupportsRead[Buffer] | Iterable[Buffer]] = None, 
     boundary: None | str = None, 
-) -> tuple[dict, Iterator[bytes | bytearray | memoryview]]:
+) -> tuple[dict, Iterator[Buffer]]:
     if not boundary:
         boundary = uuid4().bytes.hex()
     headers = {"Content-Type": f"multipart/form-data; boundary={boundary}"}
 
-    def encode_data(data) -> Iterator[bytes | bytearray | memoryview]:
+    def encode_data(data) -> Iterator[Buffer]:
+        if not data:
+            return
         if isinstance(data, Mapping):
             data = ItemsView(data)
         for name, value in data:
@@ -105,13 +106,15 @@ def encode_multipart_data(
             yield ensure_bytes(value)
             yield b"\r\n"
 
-    def encode_files(files) -> Iterator[bytes | bytearray | memoryview]:
+    def encode_files(files) -> Iterator[Buffer]:
+        if not files:
+            return
         if isinstance(files, Mapping):
             files = ItemsView(files)
         for name, file in files:
             yield boundary_line
             yield b'Content-Disposition: form-data; name="%s"\r\nContent-Type: application/octet-stream\r\n\r\n' % bytes(quote(name), "ascii")
-            if isinstance(file, (bytes, bytearray, memoryview)):
+            if isinstance(file, Buffer):
                 yield file
             elif hasattr(file, "read"):
                 yield from bio_chunk_iter(file)
@@ -124,18 +127,17 @@ def encode_multipart_data(
 
 
 def encode_multipart_data_async(
-    data: Mapping[str, Any], 
-    files: Mapping[str, bytes | bytearray | memoryview | 
-                        SupportsRead[bytes] | SupportsRead[bytearray] | SupportsRead[memoryview] | 
-                        Iterable[bytes] | Iterable[bytearray] | Iterable[memoryview] | 
-                        AsyncIterable[bytes] | AsyncIterable[bytearray] | AsyncIterable[memoryview]], 
+    data: None | Mapping[str, Any] = None, 
+    files: None | Mapping[str, Buffer | SupportsRead[Buffer] | Iterable[Buffer] | AsyncIterable[Buffer]] = None, 
     boundary: None | str = None, 
-) -> tuple[dict, AsyncIterator[bytes | bytearray | memoryview]]:
+) -> tuple[dict, AsyncIterator[Buffer]]:
     if not boundary:
         boundary = uuid4().bytes.hex()
     headers = {"Content-Type": f"multipart/form-data; boundary={boundary}"}
 
-    async def encode_data(data) -> AsyncIterator[bytes | bytearray | memoryview]:
+    async def encode_data(data) -> AsyncIterator[Buffer]:
+        if not data:
+            return
         if isinstance(data, Mapping):
             data = ItemsView(data)
         for name, value in data:
@@ -144,13 +146,15 @@ def encode_multipart_data_async(
             yield ensure_bytes(value)
             yield b"\r\n"
 
-    async def encode_files(files) -> AsyncIterator[bytes | bytearray | memoryview]:
+    async def encode_files(files) -> AsyncIterator[Buffer]:
+        if not files:
+            return
         if isinstance(files, Mapping):
             files = ItemsView(files)
         for name, file in files:
             yield boundary_line
             yield b'Content-Disposition: form-data; name="%s"\r\nContent-Type: application/octet-stream\r\n\r\n' % bytes(quote(name), "ascii")
-            if isinstance(file, (bytes, bytearray, memoryview)):
+            if isinstance(file, Buffer):
                 yield file
             elif hasattr(file, "read"):
                 async for b in bio_chunk_async_iter(file):
